export type MessageBody = string | string[] | (string | string[])[];

export const LANGUAGE_DEFAULT_STATUS_MESSAGE_KEYS = [
	'unableToInitializeResourceDataSource',
	'unableToFetchResourceCollection',
	'unableToFetchResource',
	'resourceIdNotGiven',
	'languageNotAcceptable',
	'encodingNotAcceptable',
	'mediaTypeNotAcceptable',
	'methodNotAllowed',
	'urlNotFound',
	'badRequest',
	'ok',
	'resourceCollectionFetched',
	'resourceFetched',
	'resourceNotFound',
	'deleteNonExistingResource',
	'unableToCreateResource',
	'unableToBindResourceDataSource',
	'unableToGenerateIdFromResourceDataSource',
	'unableToAssignIdFromResourceDataSource',
	'unableToEmplaceResource',
	'unableToSerializeResponse',
	'unableToEncodeResponse',
	'unableToDeleteResource',
	'unableToDeserializeResource',
	'unableToDecodeResource',
	'resourceDeleted',
	'unableToDeserializeRequest',
	'patchNonExistingResource',
	'unableToPatchResource',
	'invalidResourcePatch',
	'invalidResourcePatchType',
	'invalidResource',
	'resourcePatched',
	'resourceCreated',
	'resourceReplaced',
	'notImplemented',
	'provideOptions',
	'internalServerError',
] as const;

export type LanguageDefaultStatusMessageKey = typeof LANGUAGE_DEFAULT_STATUS_MESSAGE_KEYS[number];

export interface LanguageStatusMessageMap extends Record<LanguageDefaultStatusMessageKey, string> {}

export const LANGUAGE_DEFAULT_BODY_KEYS = [
	'languageNotAcceptable',
	'encodingNotAcceptable',
	'mediaTypeNotAcceptable',
] as const;

export type LanguageDefaultBodyKey = typeof LANGUAGE_DEFAULT_BODY_KEYS[number];

export interface LanguageBodyMap extends Record<LanguageDefaultBodyKey, MessageBody> {}

export interface Language<Name extends string = string> {
	name: Name,
	statusMessages: LanguageStatusMessageMap,
	bodies: LanguageBodyMap
}

export const FALLBACK_LANGUAGE = {
	name: 'en' as const,
	statusMessages: {
		unableToSerializeResponse: 'Unable To Serialize Response',
		unableToEncodeResponse: 'Unable To Encode Response',
		unableToBindResourceDataSource: 'Unable To Bind $RESOURCE Data Source',
		unableToInitializeResourceDataSource: 'Unable To Initialize $RESOURCE Data Source',
		unableToFetchResourceCollection: 'Unable To Fetch $RESOURCE Collection',
		unableToFetchResource: 'Unable To Fetch $RESOURCE',
		unableToDeleteResource: 'Unable To Delete $RESOURCE',
		languageNotAcceptable: 'Language Not Acceptable',
		encodingNotAcceptable: 'Encoding Not Acceptable',
		unableToDeserializeResource: 'Unable To Deserialize $RESOURCE',
		unableToDecodeResource: 'Unable To Decode $RESOURCE',
		mediaTypeNotAcceptable: 'Media Type Not Acceptable',
		methodNotAllowed: 'Method Not Allowed',
		urlNotFound: 'URL Not Found',
		badRequest: 'Bad Request',
		ok: 'OK',
		provideOptions: 'Provide Options',
		resourceCollectionFetched: '$RESOURCE Collection Fetched',
		resourceFetched: '$RESOURCE Fetched',
		resourceNotFound: '$RESOURCE Not Found',
		deleteNonExistingResource: 'Delete Non-Existing $RESOURCE',
		resourceDeleted: '$RESOURCE Deleted',
		unableToDeserializeRequest: 'Unable To Deserialize Request',
		patchNonExistingResource: 'Patch Non-Existing $RESOURCE',
		unableToPatchResource: 'Unable To Patch $RESOURCE',
		invalidResourcePatch: 'Invalid $RESOURCE Patch',
		invalidResourcePatchType: 'Invalid $RESOURCE Patch Type',
		invalidResource: 'Invalid $RESOURCE',
		resourcePatched: '$RESOURCE Patched',
		resourceCreated: '$RESOURCE Created',
		resourceReplaced: '$RESOURCE Replaced',
		unableToGenerateIdFromResourceDataSource: 'Unable To Generate ID From $RESOURCE Data Source',
		unableToAssignIdFromResourceDataSource: 'Unable To Assign ID From $RESOURCE Data Source',
		unableToEmplaceResource: 'Unable To Emplace $RESOURCE',
		resourceIdNotGiven: '$RESOURCE ID Not Given',
		unableToCreateResource: 'Unable To Create $RESOURCE',
		notImplemented: 'Not Implemented',
		internalServerError: 'Internal Server Error',
	},
	bodies: {
		languageNotAcceptable: [],
		encodingNotAcceptable: [],
		mediaTypeNotAcceptable: []
	},
} satisfies Language;
